import json
import os.path
import socket
from argparse import ArgumentParser, Namespace
from typing import Optional

from prettytable import PrettyTable

from alarmix.exceptions import AlarmDaemonIsNotRunning
from alarmix.schema import InfoList, RequestAction, TimeMessageClient, When
from alarmix.utils import SOCKET_NAME, parse_relative_time


def send_message(
    socket_addr: str,
    action: RequestAction,
    when: When = When.auto,
    time_str: Optional[str] = None,
    full_list: bool = False,
) -> str:
    """
    Communicate with alarm server running on socket-file.
    """
    if not os.path.exists(socket_addr):
        raise AlarmDaemonIsNotRunning()
    msg_obj = TimeMessageClient(
        time=time_str, when=when, action=action, full_list=full_list
    )
    with socket.socket(socket.AF_UNIX, socket.SOCK_STREAM) as sock:
        sock.connect(socket_addr)
        msg = msg_obj.json().encode("utf-8")
        sock.sendall(msg)
        data = sock.recv(1024)
        return data.decode("utf-8")


def print_alarms(socket_addr: str, full_list: bool) -> str:
    alarms_list_str = send_message(
        socket_addr=socket_addr, action=RequestAction.list, full_list=full_list
    )
    table = PrettyTable(field_names=["alarm time", "remaining time", "when"])
    alarms_list = InfoList(**json.loads(alarms_list_str))
    if len(alarms_list.alarms) == 0:
        return "No alarms found"
    for alarm in alarms_list.alarms:
        table.add_row([str(alarm.time), alarm.remaining, alarm.when])
    return table.get_string()


def parse_args() -> Namespace:
    arg_parse = ArgumentParser()
    subparsers = arg_parse.add_subparsers(dest="namespace")
    add_parser = subparsers.add_parser("add")
    stop_parser = subparsers.add_parser("stop")
    delete_parser = subparsers.add_parser("delete")
    arg_parse.add_argument(
        "-f",
        "--full",
        default=False,
        dest="full",
        action="store_true",
        help="Print information about all existing alarms",
    )
    for parser in (arg_parse, add_parser, stop_parser, delete_parser):
        parser.add_argument(
            "-s",
            "--socket",
            type=str,
            default=SOCKET_NAME,
            help="Socket path to communicate with daemon",
        )
    for parser in (add_parser, delete_parser):
        parser.add_argument(
            "time",
            type=str,
            nargs="*",
            help="List of times in format {hours}:{minutes} separated by spaces",
        )
        parser.add_argument(
            "-w", "--when", default=When.auto, type=When, choices=list(When)
        )
    return arg_parse.parse_args()


def main() -> None:
    args = parse_args()
    try:
        answer = str()
        if args.namespace is None:
            answer = print_alarms(socket_addr=args.socket, full_list=args.full)
        elif args.namespace == "add":
            for time_str in args.time:
                time_str = parse_relative_time(time_str)
                answer = send_message(
                    socket_addr=args.socket,
                    action=RequestAction.add,
                    time_str=time_str,
                    when=args.when,
                )
        elif args.namespace == "delete":
            for time_str in args.time:
                answer = send_message(
                    socket_addr=args.socket,
                    action=RequestAction.delete,
                    time_str=time_str,
                    when=args.when,
                )
        elif args.namespace == "stop":
            answer = send_message(socket_addr=args.socket, action=RequestAction.stop)
        print(answer)
    except AlarmDaemonIsNotRunning:
        print("Are you sure that timer daemon is running.")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['alarmix', 'alarmix.client', 'alarmix.daemon']

package_data = \
{'': ['*']}

install_requires = \
['loguru>=0.5.3,<0.6.0', 'prettytable>=2.0.0,<3.0.0', 'pydantic>=1.7.2,<2.0.0']

entry_points = \
{'console_scripts': ['alarmc = alarmix.client.main:main',
                     'alarmd = alarmix.daemon.main:main']}

setup_kwargs = {
    'name': 'alarmix',
    'version': '0.5.4',
    'description': 'Alarm manager server and client',
    'long_description': '|py_versions| |build_statuses| |pypi_versions|\n\n.. |py_versions| image:: https://img.shields.io/pypi/pyversions/alarmix?style=flat-square\n    :alt: python versions\n\n.. |build_statuses| image:: https://img.shields.io/github/workflow/status/s3rius/alarmix/Release%20python%20package?style=flat-square\n    :alt: build status\n\n.. |pypi_versions| image:: https://img.shields.io/pypi/v/alarmix?style=flat-square\n    :alt: pypi version\n    :target: https://pypi.org/project/fastapi-template/\n\n.. image:: https://raw.githubusercontent.com/s3rius/alarmix/master/logo.png\n    :alt: logo\n    :align: center\n\n===============\nInstallation\n===============\n\n.. code-block:: bash\n\n    python -m pip install alarmix\n\n⚠️ `MPV <https://mpv.io/>`_ must be installed and accessible ⚠️\n\nAt first, you need to start alarmd daemon:\n\n.. code-block:: bash\n\n    alarmd --sound "path/to/sound"\n\nThen you can manage your alarms with `alarmc` command.\n\n.. code-block:: bash\n\n    alarmc # Show scheduled alarms for today\n    alarmc -f # Show all scheduled alarms\n    alarmc stop # Stop buzzing alarm\n    alarmc add 20:00 19:30 14:00 # Add alarms\n    alarmc add +30 +2:40 # Add alarms with relative time\n    alarmc delete 20:00 # Remove alarm from schedule\n    alarmc\n\n    alarmc -h # Show help\n',
    'author': 'Pavel Kirilin',
    'author_email': 'win10@list.ru',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/s3rius/alarmix',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
